<?php
namespace Ep\Woo\Form;

use Ep\Woo\Entity\Product\Grouped;
use Ep\Woo\Form\Element\AttributeList;
use Zend\Form\FieldsetInterface;
use Zend\Form\Form;

/**
 * Obrazac za dodavanje proizvoda u košaricu.
 *
 * @package Ep\Woo\Form
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class AddToCart extends Form
{
    /**
     * Inicijalizacija.
     */
    public function init()
    {
        parent::init();

        $this->setAttribute('id', 'atc_' . substr(md5(rand() . time() * 24), 0, 8));

        $this->add([
            'type' => 'submit',
            'name' => 'add-to-cart',
            'options' => [
                'label' => __('Add to cart', 'woocommerce'),
            ],
            'attributes' => [
                'class' => 'add-to-cart__button btn-alt',
            ],
        ]);
    }

    /**
     * @param \WC_Product $object
     *
     * @return \Zend\Form\Fieldset|\Zend\Form\FieldsetInterface
     */
    public function setObject($object)
    {
        if (!$object instanceof \WC_Product) {
            throw new \RuntimeException("Objekt mora biti klase \WC_Product");
        }

        $this->setAttribute('data-product-id', $object->get_id());
        $this->setAttribute('data-product-type', $object->get_type());

        if ($this->has('quantity')) {
            $this->remove('quantity');
        }

        if ($object instanceof \WC_Product_Variable) {
            $this->setVariableProduct($object);
        }

        if ($object instanceof Grouped) {
            $this->setGroupedProduct($object);
        } else if ($object->is_purchasable()) {
            $this->add([
                'type' => 'woo.element.quantity',
                'name' => 'quantity',
                'options' => [
                    'label' => __('Quantity', 'woocommerce'),
                ],
                'attributes' => [
                    'value' => $object->get_min_purchase_quantity(),
                    'min' => $object->get_min_purchase_quantity(),
                    'max' => $object->get_max_purchase_quantity(),
                    'step' => 1,
                ],
            ]);
        }

        $this->add([
            'type' => 'woo.element.productenquiry',
            'name' => 'enquiry',
            'attributes' => [
                'class' => 'btn btn-alt product-enquiry add-to-cart__button',
                'data-product-name' => $object->get_name(),
                'data-product-sku' => $object->get_sku(),
            ],
        ]);

        return parent::setObject($object);
    }

    /**
     * @param Grouped $product
     */
    protected function setGroupedProduct(Grouped $product)
    {
        if ($this->has('children')) {
            $this->remove('children');
        }

        $this->add([
            'type' => 'fieldset',
            'name' => 'children',
        ]);

        /** @var FieldsetInterface $children */
        $children = $this->get('children');

        foreach ($product->getChildren() as $child) {
            if (!$child->is_purchasable()) {
                continue;
            }

            $old = $GLOBALS['product'];
            $GLOBALS['product'] = $child;

            ob_start();
            woocommerce_template_loop_price();
            $price = ob_get_clean();

            $GLOBALS['product'] = $old;

            $children->add([
                'type' => 'woo.element.quantity',
                'name' => sprintf('quantity[%d]', $child->get_id()),
                'options' => [
                    'product_id' => $child->get_id(),
                    'product_name' => $child->get_name(),
                    'product_price' => $price,
                    'product_stock' => wc_get_stock_html($child),
                    'product_url' => get_permalink($child->get_id()),
                    'product_visible' => $child->is_visible(),
                ],
                'attributes' => [
                    'value' => 0,
                    'min' => 0,
                    'max' => $child->get_max_purchase_quantity(),
                    'step' => 1,

                    'data-product-id' => $child->get_id(),
                ],
            ]);
        }
    }

    /**
     * @param \WC_Product_Variable $product
     *
     * @return void|\Zend\Form\Element|\Zend\Form\ElementInterface
     */
    protected function setVariableProduct(\WC_Product_Variable $product)
    {
        if ($this->has('attributes')) {
            $this->remove('attributes');
        }

        $this->add([
            'type' => 'woo.element.attributelist',
            'name' => 'attributes',
        ]);

        /** @var AttributeList $attributes */
        $attributes = $this->get('attributes');
        $attributes->setProduct($product);

        $variations = [];

        foreach ($product->get_available_variations() as $variation) {
            $key = implode('|', $variation['attributes']);
            $variations[$key] = $variation;
        }

        $this->setAttribute('data-product-variations', $variations);
    }
}